package com.sundata.internalevaluation.calc.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.calcite.adapter.file.JsonScannableTable;
import org.apache.calcite.jdbc.CalciteConnection;
import org.apache.calcite.schema.SchemaPlus;
import org.apache.calcite.schema.impl.AbstractSchema;
import org.apache.calcite.util.Sources;

import java.sql.*;

/**
 * Created by IntelliJ IDEA.
 *
 * @author JoeLazy
 * @date 2025-02-18 11:44:07
 * @description: json转虚拟表
 */
public class JsonToCalciteUtil {

    private Connection connection = null;

    private Statement statement = null;

    private ResultSet resultSet = null;

    public ResultSet jsonToResultSet(String jsonStr, String sql) throws SQLException, JsonProcessingException {
        // 1. 使用 Jackson 解析 JSON 数据
        ObjectMapper objectMapper = new ObjectMapper();
        JsonNode rootNode = objectMapper.readTree(jsonStr);

        //2. 创建 Calcite Schema
        connection = DriverManager.getConnection("jdbc:calcite:");
        CalciteConnection calciteConnection = connection.unwrap(CalciteConnection.class);

        SchemaPlus rootSchema = calciteConnection.getRootSchema();
        rootSchema.add("credite", new AbstractSchema());

        // 注册 JSON 数据为表
        rootNode.fields().forEachRemaining(entry -> {
            String tableName = entry.getKey();
            JsonNode tableData = entry.getValue();
            rootSchema.add(tableName, new JsonScannableTable(Sources.of(tableData.toString())));
        });

        // 4. 执行 SQL 查询
        statement = connection.createStatement();
        return resultSet = statement.executeQuery(sql);
    }

    public void closeResource() throws SQLException {
        // 关闭连接
        if (resultSet != null) {
            resultSet.close();
        }
        if (statement != null) {
            statement.close();
        }
        if (connection != null) {
            connection.close();
        }
    }

}
