package com.sundata.internalevaluation.conf;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.apache.calcite.DataContext;
import org.apache.calcite.adapter.file.JsonScannableTable;
import org.apache.calcite.jdbc.CalciteConnection;
import org.apache.calcite.linq4j.Enumerable;
import org.apache.calcite.linq4j.Linq4j;
import org.apache.calcite.rel.type.RelDataType;
import org.apache.calcite.rel.type.RelDataTypeFactory;
import org.apache.calcite.schema.SchemaPlus;
import org.apache.calcite.schema.impl.AbstractSchema;
import org.apache.calcite.schema.impl.AbstractTable;
import org.apache.calcite.sql.type.SqlTypeName;
import org.apache.calcite.util.Pair;
import org.apache.calcite.util.Sources;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.*;

public class JsonToCalciteExample {

    public static SchemaPlus jsonTotable(String schemaName, String jsonDate)  {
        // 1. JSON 数据

        try{
            // 2. 使用 Jackson 解析 JSON 数据
            ObjectMapper objectMapper = new ObjectMapper();
            JsonNode rootNode = objectMapper.readTree(jsonDate);

            // 3. 创建 Calcite Schema
            Connection connection = DriverManager.getConnection("jdbc:calcite:");
            CalciteConnection calciteConnection = connection.unwrap(CalciteConnection.class);

            SchemaPlus rootSchema = calciteConnection.getRootSchema();

            SchemaPlus schema = rootSchema.add(schemaName,new AbstractSchema());

            // 注册 JSON 数据为表
            rootNode.fields().forEachRemaining(entry -> {
                String tableName = entry.getKey();
                JsonNode tableData = entry.getValue();
                schema.add(tableName, new JsonScannableTable(Sources.of(tableData.toString())));
            });
            // 4. 执行 SQL 查询
            Statement statement = connection.createStatement();
            String sql = """
            SELECT "test"."input"."method"   FROM "test"."input" 
        """;

            ResultSet resultSet = statement.executeQuery(sql);
            while (resultSet.next()){
                String method=resultSet.getString("method");
                System.out.println(method);
            }
            // 打印查询结果
           /*while (resultSet.next()) {
            System.out.printf("Order ID: %d, Amount: %.2f, Name: %s%n",
                    resultSet.getInt("user_id"),
                    resultSet.getDouble("amount"),
                    resultSet.getString("name"));

           }*/

            // 关闭连接
            resultSet.close();
            statement.close();
            connection.close();
            return rootSchema;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }

    // 自定义表类实现
    static class JsonTable extends AbstractTable {
        private final List<Map<String, Object>> rows = new ArrayList<>();

        public JsonTable(JsonNode data) {
            for (JsonNode row : data) {
                Map<String, Object> map = new HashMap<>();
                row.fieldNames().forEachRemaining(field -> {
                    map.put(field, row.get(field).isNumber()
                            ? row.get(field).numberValue()
                            : row.get(field).asText());
                });
                rows.add(map);
            }
        }

        @Override
        public RelDataType getRowType(RelDataTypeFactory typeFactory) {
            // 动态推断字段类型
            if (rows.isEmpty()) {
                return typeFactory.createStructType(Collections.emptyList());
            }

            Map<String, Object> firstRow = rows.get(0);
            List<Pair<String, RelDataType>> fields = new ArrayList<>();
            for (Map.Entry<String, Object> entry : firstRow.entrySet()) {
                SqlTypeName sqlType = (entry.getValue() instanceof Number)
                        ? SqlTypeName.DECIMAL
                        : SqlTypeName.VARCHAR;
                fields.add(Pair.of(entry.getKey(), typeFactory.createSqlType(sqlType)));
            }

            return typeFactory.createStructType(fields);
        }

        public Enumerable<Object[]> scan(DataContext root) {
            List<Object[]> result = new ArrayList<>();
            for (Map<String, Object> row : rows) {
                result.add(row.values().toArray());
            }
            return Linq4j.asEnumerable(result);
        }
    }
}
