package com.sundata.internalevaluation.configuration.service;

import com.sundata.common.util.JsonUtil;
import com.sundata.internalevaluation.configuration.mapper.IndexConfigMapper;
import com.sundata.internalevaluation.configuration.model.IndexConfigModel;
import com.sundata.internalevaluation.configuration.model.IndexSourceModel;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Random;
import java.util.UUID;

@Service
public class IndexConfigService {

    @Autowired
    IndexConfigMapper indexConfigMapper;

    /**
     * @author: 张艳杰
     * @methodsName: getDataList
     * @description: 指标配置初始化
     * @param: IndexConfigModel model
     * @return: List<IndexConfigModel>
     * @throws:
     */
    public List<IndexConfigModel> getDataList(IndexConfigModel model) {
        return indexConfigMapper.getDataList(model);
    }

    /**
     * @author: 张艳杰
     * @methodsName: createData
     * @description: 指标配置新增
     * @param: IndexConfigModel model
     * @return:
     * @throws:
     */
    public void createData(IndexConfigModel model) {
        indexConfigMapper.createData(model);
        if (null != model.getIndexDataSource() && model.getIndexDataSource().size() >0) {
            indexConfigMapper.deleteIndexSource(model.getIndexNo());
            for ( IndexSourceModel m : model.getIndexDataSource()) {
                m.setIndexNo(model.getIndexNo());
                m.setIndexSourceNo(UUID.randomUUID().toString());
            }
            indexConfigMapper.createIndexSource(model.getIndexDataSource());
        }
    }

    /**
     * @author: 张艳杰
     * @methodsName: deleteRows
     * @description: 指标配置删除
     * @param: IndexConfigModel model
     * @return:
     * @throws:
     */
    public void deleteRows(IndexConfigModel model) {
        indexConfigMapper.deleteRows(model);
        indexConfigMapper.deleteIndexSource(model.getIndexNo());
    }

    /**
     * @author: 张艳杰
     * @methodsName: updateExistData
     * @description: 指标配置修改
     * @param: IndexConfigModel model
     * @return:
     * @throws:
     */
    public void updateExistData(IndexConfigModel model) {
        indexConfigMapper.updateExistData(model);
        if (null != model.getIndexDataSource() && model.getIndexDataSource().size() >0) {
            indexConfigMapper.deleteIndexSource(model.getIndexNo());
            for ( IndexSourceModel m : model.getIndexDataSource()) {
                m.setIndexNo(model.getIndexNo());
                m.setIndexSourceNo(UUID.randomUUID().toString());
            }
            indexConfigMapper.createIndexSource(model.getIndexDataSource());
        }
    }

    /**
     * @author: 张艳杰
     * @methodsName: selectDetailData
     * @description: 指标配置查询
     * @param: IndexConfigModel model
     * @return:
     * @throws:
     */
    public IndexConfigModel selectDetailData(IndexConfigModel model) {
        IndexConfigModel selectDataSource = indexConfigMapper.selectDetailData(model);

        List<IndexSourceModel> sourceList = indexConfigMapper.getIndexSourceList(selectDataSource);
        selectDataSource.setIndexDataSource(sourceList);
        return selectDataSource;
    }

    public List<IndexSourceModel> getIndexSourceList(IndexConfigModel indexConfigModel) {
        return indexConfigMapper.getIndexSourceList(indexConfigModel);
    }

    public List<IndexConfigModel> selectIndexListByRuleNo(String ruleNo) {
        return indexConfigMapper.selectIndexListByRuleNo(ruleNo);
    }
}
